"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = css;

var _stylesheet = require("./stylesheet");

var _cache = require("./cache");

var _utils = require("../utils");

function isValidRule(prop) {
  return (0, _utils.hasDOM)() ? prop in document.body.style : true;
}

function toSelector(className) {
  var pseudo = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
  var children = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : '';
  var wrap = arguments.length > 3 ? arguments[3] : undefined;
  var dotted = '.' + className;
  var specific = wrap ? dotted + dotted : dotted;
  var parent = specific + pseudo;
  return Boolean(children) ? children.split(',').map(function (child) {
    return "".concat(parent, " ").concat(child.trim());
  }).join(', ') // prettier-ignore
  : parent;
}

function toRule(className, value, prop, pseudo, children, wrap) {
  var selector = toSelector(className, pseudo, children, wrap);
  var rule = "".concat(selector, " { ").concat(prop, ": ").concat(value, "; }");
  return wrap ? "".concat(wrap, " { ").concat(rule, " }") : rule;
}

function createRule(value, prop, pseudo, children, wrap) {
  var className = (0, _utils.uid)('r');
  var rule = toRule(className, value, prop, pseudo, children, wrap);
  (0, _stylesheet.insertRule)(rule);
  return className;
}

function findOrCreateRule(value, prop, pseudo, children, wrap) {
  if (!(0, _cache.getCache)(value, prop, pseudo, children, wrap)) {
    var className = createRule(value, prop, pseudo, children, wrap);
    (0, _cache.setCache)(className, value, prop, pseudo, children, wrap);
  }

  return (0, _cache.getCache)(value, prop, pseudo, children, wrap);
}

function css(style, pseudo, children, wrap) {
  return Object.keys(style).filter(isValidRule).map(function (prop) {
    return findOrCreateRule(style[prop], prop, pseudo, children, wrap);
  });
}
//# sourceMappingURL=css.js.map