"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.to = to;
exports.alias = alias;
exports.pseudo = pseudo;
exports.combineTransforms = combineTransforms;
exports.default = computed;

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _objectSpread3 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _utils = require("../../utils");

function to(rule) {
  if ((0, _utils.isFn)(rule)) {
    return rule;
  } else if ((0, _utils.isStr)(rule)) {
    return function (value) {
      return (0, _defineProperty2.default)({}, rule, value);
    };
  } else if (Array.isArray(rule)) {
    return function (value) {
      return rule.reduce(function (result, prop) {
        return (0, _objectSpread3.default)({}, result, (0, _defineProperty2.default)({}, prop, value));
      }, {});
    };
  }
}

function alias(rules) {
  var aliased = {};
  (0, _utils.forEach)(rules, function (rule, prop) {
    return aliased[prop] = to(rule);
  });
  return aliased;
}

function pseudo(name) {
  return function (value) {
    return {
      $pseudo: (0, _defineProperty2.default)({}, name, value)
    };
  };
}

function combineTransforms(first) {
  for (var _len = arguments.length, transforms = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    transforms[_key - 1] = arguments[_key];
  }

  return function (root) {
    return function (value, props) {
      var rest = transforms.reduce(function (next, transform) {
        return function (prev) {
          return transform(prev, props, next);
        };
      }, to(root));
      return first(value, props, rest);
    };
  };
}

function computed(props, transforms) {
  var rules = {};
  var transform = combineTransforms.apply(void 0, (0, _toConsumableArray2.default)(transforms));
  (0, _utils.forEach)(props, function (rule, prop) {
    return rules[prop] = transform(rule === true ? prop : rule);
  });
  return rules;
}
//# sourceMappingURL=computed.js.map