"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createSystem;

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _utils = require("./utils");

function getName(_ref) {
  var displayName = _ref.displayName,
      As = _ref.as;
  var asDName = As && As.displayName;
  var asName = As && As.name;
  var tag = (0, _utils.isStr)(As) && As;
  var name = (0, _utils.capitalize)(displayName || asDName || asName || tag || 'div');
  return name[0] === 'Σ' ? name : 'Σ' + name;
}

function separateSystem(props, rules) {
  var sys = {};
  var other = {};
  (0, _utils.forEach)(props, function (value, prop) {
    if (rules[prop]) {
      sys[prop] = value;
    } else {
      other[prop] = value;
    }
  });
  return [sys, other];
} // execute the functional default props by passing them current props


function computeDefaults(defaults, props) {
  var computedDefaults = {};
  var allProps = (0, _utils.merge)(defaults, props);
  (0, _utils.forEach)(defaults, function (value, prop) {
    computedDefaults[prop] = prop !== 'as' && (0, _utils.isFn)(value) ? value(allProps) : value;
  });
  return computedDefaults;
}

function computeSystem(sys, props, rules, omit) {
  var computedProps = {};
  (0, _utils.forEach)(sys, function (value, prop) {
    if (prop === omit) {
      return computedProps[prop] = value;
    }

    var rule = rules[prop];
    var isRuleFn = (0, _utils.isFn)(rule);
    var result = isRuleFn ? rule(value, props) : value;
    var isResultObj = (0, _utils.isObj)(result);

    if (isResultObj) {
      // apply deeper but ignore self referencing rules
      result = computeSystem(result, props, rules, prop);
    }

    if (isRuleFn && isResultObj) {
      computedProps = (0, _utils.merge)(computedProps, result);
    } else {
      computedProps[prop] = result;
    }
  });
  return computedProps;
}

function createComputeProps(uniqueID, defaults, rules, sideEffect) {
  return function (props) {
    var computedDefaults = computeDefaults(defaults, props);
    var baseProps = (0, _utils.merge)(computedDefaults, props);

    var _separateSystem = separateSystem(baseProps, rules),
        _separateSystem2 = (0, _slicedToArray2.default)(_separateSystem, 2),
        sysProps = _separateSystem2[0],
        passedProps = _separateSystem2[1];

    var computedProps = computeSystem(sysProps, baseProps, rules);

    var _separateSystem3 = separateSystem(computedProps, rules),
        _separateSystem4 = (0, _slicedToArray2.default)(_separateSystem3, 2),
        cmpSysProps = _separateSystem4[0],
        cmpPassedProps = _separateSystem4[1];

    var allPassedProps = (0, _utils.merge)(cmpPassedProps, passedProps);
    var sideEffectProps = sideEffect(uniqueID, cmpSysProps, allPassedProps, rules); // prettier-ignore

    var _merge = (0, _utils.merge)(allPassedProps, sideEffectProps),
        omit = _merge.omit,
        nextProps = (0, _objectWithoutProperties2.default)(_merge, ["omit"]);

    return _utils.omit.apply(void 0, [nextProps].concat((0, _toConsumableArray2.default)([].concat(omit))));
  };
}

function createSystem(renderer) {
  var sideEffect = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : _utils.noop;
  var methods = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : _utils.noop;
  return function system() {
    var rules = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    function createComponent(Root) {
      var defaults = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      var wrap = arguments.length > 2 ? arguments[2] : undefined;
      var uniqueID = (0, _utils.uid)('c');
      var computeProps = createComputeProps(uniqueID, defaults, rules, sideEffect);
      var Component = renderer(Root, computeProps);
      var System = wrap ? wrap(Component) : Component;

      System.system = function extendSystem(extra) {
        var System = system((0, _utils.merge)(rules, extra), sideEffect, methods).compose(wrap).extend(defaults);
        return Root ? System.with(Root) : System;
      };

      System.extend = function extendProps(props) {
        return createComponent(Root, (0, _utils.merge)(defaults, props), wrap);
      };

      System.with = function replaceRoot(NewRoot) {
        return createComponent(NewRoot, defaults, wrap);
      };

      System.compose = function addWrappers() {
        for (var _len = arguments.length, hocs = new Array(_len), _key = 0; _key < _len; _key++) {
          hocs[_key] = arguments[_key];
        }

        return createComponent(Root, defaults, _utils.compose.apply(void 0, [wrap].concat(hocs))); // prettier-ignore
      };

      System.compute = function computeValues(values) {
        return computeSystem(values, defaults, rules);
      };

      System.uid = uniqueID;
      System.defaults = defaults;
      System.displayName = getName(defaults);
      (0, _utils.forEach)(methods(System), function (method, name) {
        System[name] = method;
      });
      return System;
    }

    return createComponent();
  };
}
//# sourceMappingURL=system.js.map