"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.noop = noop;
exports.uid = uid;
exports.forEach = forEach;
exports.capitalize = capitalize;
exports.kebabCase = kebabCase;
exports.isObj = isObj;
exports.isDef = isDef;
exports.isStr = isStr;
exports.isFn = isFn;
exports.uniq = uniq;
exports.get = get;
exports.set = set;
exports.omit = omit;
exports.compose = compose;
exports.merge = merge;
exports.hasDOM = hasDOM;
exports.cs = cs;

var _toArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toArray"));

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

var _nanoid = _interopRequireDefault(require("nanoid"));

function noop() {}

function uid(root) {
  return root + (0, _nanoid.default)(5);
}

function forEach(obj, sideEffect) {
  Object.keys(obj).forEach(function (key) {
    return sideEffect(obj[key], key);
  });
}

function capitalize(str) {
  return str[0].toUpperCase() + str.slice(1);
}

function kebabCase() {
  var str = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
  return str.replace(/[A-Z]/g, '-$&').toLowerCase();
}

function isObj(val) {
  return (0, _typeof2.default)(val) === 'object' && isDef(val) && !Array.isArray(val);
}

function isDef(val) {
  return val === 0 || Boolean(val);
}

function isStr(val) {
  return typeof val === 'string';
}

function isFn(val) {
  return typeof val === 'function';
}

function uniq(value, index, arr) {
  return arr.lastIndexOf(value) === index;
}

function get(obj, _ref) {
  var _ref2 = (0, _toArray2.default)(_ref),
      first = _ref2[0],
      after = _ref2.slice(1);

  return isDef(first) && isDef(obj) ? get(obj[first], after) : obj;
}

function set(obj, _ref3, value) {
  var _ref4 = (0, _toArray2.default)(_ref3),
      first = _ref4[0],
      after = _ref4.slice(1);

  if (isDef(first) && isDef(obj)) {
    if (after.length === 0) {
      obj[first] = value;
    } else {
      obj[first] = obj[first] || {};
      set(obj[first], after, value);
    }
  }

  return obj;
}

function omit(obj) {
  for (var _len = arguments.length, keys = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    keys[_key - 1] = arguments[_key];
  }

  var result = {};
  forEach(obj, function (v, k) {
    return !keys.includes(k) && (result[k] = v);
  });
  return result;
}

function compose() {
  for (var _len2 = arguments.length, functions = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
    functions[_key2] = arguments[_key2];
  }

  var valid = functions.filter(isFn);
  return valid.length > 0 ? function (param) {
    return valid.reduce(function (composed, fn) {
      return fn(composed);
    }, param);
  } : null;
}

function merge(target, source) {
  if (!source || target === source) {
    return target;
  }

  var result = target ? merge(null, target) : {};
  forEach(source, function (sourceValue, key) {
    var targetValue = result[key];
    result[key] = isObj(targetValue) && isObj(sourceValue) ? merge(targetValue, sourceValue) : sourceValue;
  });
  return result;
}

function hasDOM() {
  return typeof document !== 'undefined';
}

function cs() {
  for (var _len3 = arguments.length, classNames = new Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
    classNames[_key3] = arguments[_key3];
  }

  return classNames.filter(isDef).map(String).map(function (v) {
    return v.startsWith('.') ? v.slice(1) : v;
  }).filter(uniq).join(' ');
}
//# sourceMappingURL=utils.js.map